@@ -1,4 +1,5 @@
 import os
+import shlex
 import stat
 import subprocess
 import sys
@@ -20,6 +21,16 @@ pytestmark = pytest.mark.skipif(
 )
 
 
+def _check_output(cmd, *args, **kwargs):
+    if isinstance(cmd, str):
+        cmd = shlex.split(cmd)
+    kwargs.setdefault("stderr", subprocess.STDOUT)
+    output = subprocess.check_output(cmd, *args, **kwargs)
+    if not isinstance(output, str):
+        output = output.decode("utf-8")
+    return output
+
+
 def _cleanup_process(proc):
     proc.wait()
     # Make sure all the fds get closed.
@@ -48,20 +59,19 @@ def test_shutdown_sock_server_integration(jp_unix_socket_file):
 
     assert complete, "did not find socket URL in stdout when launching notebook"
 
-    socket_path = encoded_sock_path.encode()
-    assert socket_path in subprocess.check_output(["jupyter-server", "list"])
+    assert encoded_sock_path in _check_output("jupyter-server list")
 
     # Ensure umask is properly applied.
     assert stat.S_IMODE(os.lstat(jp_unix_socket_file).st_mode) == 0o700
 
     try:
-        subprocess.check_output(["jupyter-server", "stop"], stderr=subprocess.STDOUT)
+        _check_output("jupyter-server stop")
     except subprocess.CalledProcessError as e:
         assert "There is currently no server running on" in e.output.decode()
     else:
         raise AssertionError("expected stop command to fail due to target mis-match")
 
-    assert encoded_sock_path.encode() in subprocess.check_output(["jupyter-server", "list"])
+    assert encoded_sock_path in _check_output("jupyter-server list")
 
     # Fake out stopping the server.
     app = JupyterServerStopApp(sock=str(jp_unix_socket_file))
@@ -70,9 +80,9 @@ def test_shutdown_sock_server_integration(jp_unix_socket_file):
     app._maybe_remove_unix_socket = lambda _: _  # type: ignore
     app.start()
 
-    subprocess.check_output(["jupyter-server", "stop", jp_unix_socket_file])
+    _check_output(["jupyter-server", "stop", jp_unix_socket_file])
 
-    assert encoded_sock_path.encode() not in subprocess.check_output(["jupyter-server", "list"])
+    assert encoded_sock_path not in _check_output(["jupyter-server", "list"])
 
     _cleanup_process(p)
 
@@ -80,10 +90,7 @@ def test_shutdown_sock_server_integration(jp_unix_socket_file):
 @pytest.mark.integration_test
 def test_sock_server_validate_sockmode_type():
     try:
-        subprocess.check_output(
-            ["jupyter-server", "--sock=/tmp/nonexistent", "--sock-mode=badbadbad"],
-            stderr=subprocess.STDOUT,
-        )
+        _check_output(["jupyter-server", "--sock=/tmp/nonexistent", "--sock-mode=badbadbad"])
     except subprocess.CalledProcessError as e:
         assert "badbadbad" in e.output.decode()
     else:
@@ -93,9 +100,8 @@ def test_sock_server_validate_sockmode_type():
 @pytest.mark.integration_test
 def test_sock_server_validate_sockmode_accessible():
     try:
-        subprocess.check_output(
+        _check_output(
             ["jupyter-server", "--sock=/tmp/nonexistent", "--sock-mode=0444"],
-            stderr=subprocess.STDOUT,
         )
     except subprocess.CalledProcessError as e:
         assert "0444" in e.output.decode()
@@ -105,7 +111,7 @@ def test_sock_server_validate_sockmode_accessible():
 
 def _ensure_stopped(check_msg="There are no running servers"):
     try:
-        subprocess.check_output(["jupyter-server", "stop"], stderr=subprocess.STDOUT)
+        _check_output(["jupyter-server", "stop"])
     except subprocess.CalledProcessError as e:
         assert check_msg in e.output.decode()
     else:
@@ -135,19 +141,15 @@ def test_stop_multi_integration(jp_unix_socket_file, jp_http_port):
     time.sleep(3)
 
     shutdown_msg = MSG_TMPL.format(jp_http_port)
-    assert shutdown_msg in subprocess.check_output(["jupyter-server", "stop"]).decode()
+    assert shutdown_msg in _check_output(["jupyter-server", "stop"])
 
     _ensure_stopped("There is currently no server running on 8888")
 
-    assert (
-        MSG_TMPL.format(jp_unix_socket_file)
-        in subprocess.check_output(["jupyter-server", "stop", jp_unix_socket_file]).decode()
+    assert MSG_TMPL.format(jp_unix_socket_file) in _check_output(
+        ["jupyter-server", "stop", jp_unix_socket_file]
     )
 
-    assert (
-        MSG_TMPL.format(TEST_PORT)
-        in subprocess.check_output(["jupyter-server", "stop", TEST_PORT]).decode()
-    )
+    assert MSG_TMPL.format(TEST_PORT) in _check_output(["jupyter-server", "stop", TEST_PORT])
 
     _ensure_stopped()
 
@@ -169,11 +171,11 @@ def test_launch_socket_collision(jp_unix_socket_file):
 
     # Try to start a server bound to the same UNIX socket.
     try:
-        subprocess.check_output(cmd, stderr=subprocess.STDOUT)
+        _check_output(cmd)
     except subprocess.CalledProcessError as cpe:
         assert check_msg in cpe.output.decode()
     except Exception as ex:
-        raise AssertionError(f"expected 'already in use' error, got '{ex}'!")
+        raise AssertionError(f"expected 'already in use' error, got '{ex}'!") from ex
     else:
         raise AssertionError("expected 'already in use' error, got success instead!")
 
@@ -238,12 +240,12 @@ def test_jupyter_server_apps(jp_environ):
     app.json = False
     app.start()
 
-    app = JupyterServerStopApp()
-    app.initialize([])
-    app.port = port
+    stop_app = JupyterServerStopApp()
+    stop_app.initialize([])
+    stop_app.port = port
     while 1:
         try:
-            app.start()
+            stop_app.start()
             break
         except ConnectionRefusedError:
             time.sleep(0.1)
